# -*- coding: utf-8 -*-

import unittest
from laonlp.translate import word_dictionary


class TestTagPackage(unittest.TestCase):
    def test_word_dictionary(self):
        self.assertIsNotNone(word_dictionary("cat","en","lao"))


# -*- coding: utf-8 -*-
"""
Copyright 2020 - 2023 Wannaphong Phatthiyaphaibun

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from typing import List
import gensim
from huggingface_hub import hf_hub_download


class Word2Vec:
    """
    Word2Vec
    """
    def __init__(self, model: str, corpus: str="oscar"):
        """
        :param str model: model name (cbow or skip-gram)
        :param str corpus: corpus name (oscar)
        """
        self.model = model
        self.corpus = corpus
        if self.corpus not in ["oscar"]:
            raise NotImplementedError("LaoNLP doesn't support %s corpus." % self.corpus)
        self.load_model(self.model)
    
    def load_model(self, model: str):
        """
        Load Word2Vec model

        :param str model: model name (cbow or skip-gram)
        """
        if model=="cbow":
            self.model_path = hf_hub_download(repo_id="wannaphong/Lao-Word-Embedding", filename="lao_oscar_cbow_model.bin")
        elif model=="skip-gram":
            self.model_path = hf_hub_download(repo_id="wannaphong/Lao-Word-Embedding", filename="lao_oscar_skipgram_model.bin")
        else:
            raise NotImplementedError("LaoNLP doesn't support %s model." % model)
        self.model_wav2vec = gensim.models.keyedvectors.KeyedVectors.load_word2vec_format(self.model_path, binary=True, encoding='utf-8-sig', unicode_errors='ignore')
    
    def get_model(self):
        """
        Get gensim.models.keyedvectors.KeyedVectors class
        """
        return self.model_wav2vec
    
    def doesnt_match(self, words: List[str]) -> str:
        """
        Get donesn't match

        :param list words: list words

        :return: return word that donesn't match.
        :rtype: str
        """
        return self.model_wav2vec.doesnt_match(words)
    
    def most_similar_cosmul(self, positive: List[str], negative: List[str]):
        return self.model_wav2vec.most_similar_cosmul(
            positive=positive, negative=negative
        )
    
    def similarity(self, word1: str, word2: str) -> float:
        """
        Find similarity between word pairs.

        :param str word1: first word
        :param str word2: second word

        :return: return similarity
        :rtype: float
        """
        return self.model_wav2vec.similarity(word1, word2)


# -*- coding: utf-8 -*-
"""
Copyright 2020 - 2023 Wannaphong Phatthiyaphaibun

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from typing import List
from typing import FrozenSet
from laonlp.corpus.core import get_path_corpus


def lao_dictionary() -> List[str]:
    """
    This file comes from Brian Eugene Wilson, Robert Martin Campbell.
    """
    path = get_path_corpus("Lao-Dictionary.txt")
    with open(path, "r", encoding="utf-8-sig") as f:
        return [i.strip() for i in f.readlines() if i[0]!="#"]


def lao_spellcheckdict() -> List[str]:
    """
    Lao spell check dictionary
    """
    path = get_path_corpus("lo_spellcheck_dict.txt")
    with open(path, "r", encoding="utf-8-sig") as f:
        return [i.strip() for i in f.readlines() if i[0]!="#"]


def lao_wannaphongdict() -> List[str]:
    path = get_path_corpus("lao-wannaphong.txt")
    with open(path, "r", encoding="utf-8-sig") as f:
        return [i.strip() for i in f.readlines() if i[0]!="#"]


def lao_wiktionarydict() -> List[str]:
    """
    This dictionary comes from Wiktionary on 20 July 2021.
    """
    path = get_path_corpus("wiktionary-20210720.txt")
    with open(path, "r", encoding="utf-8-sig") as f:
        return [i.strip() for i in f.readlines() if i[0]!="#"]


def lao_words() -> List[str]:
    return list(set(lao_dictionary()+lao_spellcheckdict()+lao_wannaphongdict()+lao_wiktionarydict()))


def lao_stopwords() -> FrozenSet[str]:
    """
    Lao stopword list

    Return a frozenset of Lao stopwords

    :return: :class:`frozenset` containing stopwords.
    :rtype: :class:`frozenset`
    """
    path = get_path_corpus("stopwords_lao.txt")
    with open(path, "r", encoding="utf-8-sig") as fh:
        lines = fh.read().splitlines()
    lines = [line.strip() for line in lines if line.startswith("#") == False]
    return frozenset(filter(None, lines))



# -*- coding: utf-8 -*-
"""
Copyright 2020 - 2023 Wannaphong Phatthiyaphaibun

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
NUMBERS = "໑໒໓໔໕໖໗໘໙໐"
_arabic_numerals = "1234567890"
_pronunciation = [
    "ນຶ່ງ",
    "ສອງ",
    "ສາມ",
    "ສີ່",
    "ຫ້າ",
    "ຫົກ",
    "ເຈັດ",
    "ແປດ",
    "ເກົ້າ",
    "ສູນ"
]
_dict_lao_arabic = {
    i:j for i,j in zip(list(NUMBERS), list(_arabic_numerals))
}
_dict_arabic_lao = {
    i:j for i,j in zip(list(_arabic_numerals), list(NUMBERS))
}
_lao_arabic_table = str.maketrans(_dict_lao_arabic)
_arabic_lao_table = str.maketrans(_dict_arabic_lao)


def lao_digit_to_arabic_digit(text: str) -> str:
    """
    Lao digit to Arabic digit

    :param str text: lao digit text
    :return: returns a string of Arabic digit
    :rtype: str
    """
    return text.translate(_lao_arabic_table)

def arabic_digit_to_lao_digit(text: str) -> str:
    """
    Arabic digit to Lao digit

    :param str text: Arabic digit text
    :return: returns a string of Lao digit
    :rtype: str
    """
    return text.translate(_arabic_lao_table)

def number2lao(numbers: int):
    """
    Numbers to La opronunciation
    """
    # TODO
    return ""



# -*- coding: utf-8 -*-
"""
Copyright 2020 - 2023 Wannaphong Phatthiyaphaibun

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import csv
from collections import defaultdict

from laonlp.corpus import laonlp_path
from laonlp.corpus.core import get_path_corpus
corpus_path = get_path_corpus("lao-eng-dictionary.csv")
list_data=[]
with open(corpus_path,encoding="utf-8-sig") as csvfile:
    reader = csv.DictReader(csvfile)
    for row in reader:
        list_data.append(row)


def get_lao_eng()->dict:
    _w = defaultdict(list)
    for i in list_data:
        _w[i['LaoWord']].append(i['English'])
    return _w


def get_eng_lao()->dict:
    _w = defaultdict(list)
    for i in list_data:
        _w[i['English']].append(i['LaoWord'])
    return _w


def get_pronunciation()->dict:
    _w = defaultdict(list)
    for i in list_data:
        _w[i['LaoWord']].append(i['Pronunciation'])
    return _w


def get_type()->dict:
    _w = defaultdict(list)
    for i in list_data:
        _w[i['LaoWord']].append(i['Type'])
    return _w



# -*- coding: utf-8 -*-
"""
Copyright 2020 - 2023 Wannaphong Phatthiyaphaibun

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from typing import List, Tuple
from laonlp.corpus import get_path_corpus
from pythainlp.tag import PerceptronTagger


def pos_tag(
    words: List[str],
    engine: str = "perceptron",
    corpus: str = "SeqLabeling"
) -> List[Tuple[str, str]]:
    """
    Lao Part-of-speech

    We use lao corpus from https://github.com/FoVNull/SeqLabeling

    :param List[str] words:  a list of tokenized lao words
    :param str engine: engine of pos_tag (`perceptron` engine)
    :param str corpus: SeqLabeling (corpus from https://github.com/FoVNull/SeqLabeling) or yunshan_cup_2020 (corpus from https://github.com/GKLMIP/Yunshan-Cup-2020)

    :return: a list of tuples (word, POS tag)
    :rtype: List[tuple[str, str]]

    :Example:
    ::
            from laonlp.tokenize import word_tokenize
            from laonlp.tag import pos_tag

            sent = word_tokenize("ພາສາລາວໃນປັດຈຸບັນ.")
            pos_tag(sent)
            # output: [('ພາສາລາວ', 'N'), ('ໃນ', 'PRE'), ('ປັດຈຸບັນ', 'ADJ'), ('.', 'PUNCT')]
    """
    if corpus == "yunshan_cup_2020":
        _FILENAME = "ptagger_Yunshan-Cup_corpus.json"
    else:
        _FILENAME = "ptagger_SeqLabeling_corpus.json"
    _PATH = get_path_corpus(_FILENAME)
    _TAGGER = PerceptronTagger(path=_PATH)
    return _TAGGER.tag(words)



# -*- coding: utf-8 -*-

import unittest
from laonlp.tokenize import word_tokenize
from laonlp.tag import pos_tag


class TestTagPackage(unittest.TestCase):
    def test_pos_tag(self):
        self.assertIsNotNone(pos_tag(word_tokenize("ພາສາລາວໃນປັດຈຸບັນ.")))
        self.assertIsNotNone(pos_tag(word_tokenize("ພາສາລາວໃນປັດຈຸບັນ."), corpus="SeqLabeling"))
        self.assertIsNotNone(pos_tag(word_tokenize("ພາສາລາວໃນປັດຈຸບັນ."), corpus="yunshan_cup_2020"))



# -*- coding: utf-8 -*-
"""
Copyright 2020 - 2023 Wannaphong Phatthiyaphaibun

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from setuptools import find_packages, setup

with open("README.md","r",encoding="utf-8-sig") as f:
    readme = f.read()

with open("requirements.txt","r",encoding="utf-8-sig") as f:
    requirements = [i.strip() for i in f.readlines()]

setup(
    name="LaoNLP",
    version="1.0",
    description="Lao Natural Language Processing library",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Wannaphong",
    author_email="wannaphong@yahoo.com",
    url="https://github.com/wannaphong/laonlp",
    packages=find_packages(),
    test_suite="tests",
    python_requires=">=3.6",
    package_data={
        "laonlp": [
            "corpus/*",
        ]
    },
    install_requires=requirements,
    license="Apache Software License 2.0",
    zip_safe=False,
    keywords=[
        "lao",
        "NLP",
        "natural language processing",
        "text analytics",
        "text processing",
        "localization",
        "computational linguistics",
        "Lao language",
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Topic :: Text Processing",
        "Topic :: Text Processing :: General",
        "Topic :: Text Processing :: Linguistic",
    ],
    project_urls={
        "Documentation": "https://github.com/wannaphong/LaoNLP/wiki",
        "Source": "https://github.com/wannaphong/LaoNLP",
        "Bug Reports": "https://github.com/wannaphong/LaoNLP/issues",
    },
)
